/***************************************************************************
                          fractalalgorithminterface.h  -  description
                             -------------------
    begin                : Son Mai 11 2003
    copyright            : (C) 2003 by Thomas Dreibholz
    email                : dreibh@exp-math.uni-essen.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef FRACTALALGORITHMINTERFACE_H
#define FRACTALALGORITHMINTERFACE_H

#include <qptrlist.h>
#include <complex>
#include "configentry.h"


/**
  *@author Thomas Dreibholz
  */
class FractalAlgorithmInterface {
   public: 
   FractalAlgorithmInterface();
   virtual ~FractalAlgorithmInterface();
   virtual const char* getName() = 0;
   virtual const char* getSaveIdentifier() = 0;
   virtual std::complex<double> defaultC1();
   virtual std::complex<double> defaultC2();
   virtual int defaultMaxIterations();   
   virtual void configure(unsigned int         width,
                          unsigned int         height,
                          std::complex<double> c1,
                          std::complex<double> c2,                          
                          unsigned int         maxIterations);                          
   virtual unsigned int calculatePoint(const unsigned int x,
                                       const unsigned int y) = 0;

   inline QPtrList<ConfigEntry> *getConfigEntries() {return &ConfigEntries;}
   const std::complex<double> getC1() {return C1;}
   const std::complex<double> getC2() {return C2;}
   

   static FractalAlgorithmInterface* getAlgorithm(const unsigned int index);

   virtual void changeSize(int X, int Y);
   inline unsigned int* getMaxIterations() {return &MaxIterations;}

                                      
   protected:
   unsigned int         Width;
   unsigned int         Height;
   unsigned int         MaxIterations;
   std::complex<double> C1;
   std::complex<double> C2;
   double               StepX;
   double               StepY;

   QPtrList<ConfigEntry> ConfigEntries;  

   private:
   static QPtrList<FractalAlgorithmInterface> AlgorithmList;
};

#endif
