/***************************************************************************
                          fractalcalculationthread.cpp  -  description
                             -------------------
    begin                : Son Mai 11 2003
    copyright            : (C) 2003 by Thomas Dreibholz
    email                : dreibh@exp-math.uni-essen.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "fractalcalculationthread.h"

#define MIN(a,b) ((a < b) ? a : b)
#define MAX(a,b) ((a > b) ? a : b)


FractalCalculationThread::FractalCalculationThread(QObject*                   parent,
                                                   FractalAlgorithmInterface* algorithm,
                                                   ColorSchemeInterface*      cs,
                                                   FractalBuffer*             buffer,
                                                   QImage*                    image,
                                                   unsigned int               progStep)
{
   Parent        = parent;
   Algorithm     = algorithm;
   ColorScheme	 = cs;
   Buffer        = buffer;
   Image         = image;
   ProgStep      = progStep;
   MaxIterations = *Algorithm->getMaxIterations();
   Stop          = false;
}

FractalCalculationThread::~FractalCalculationThread()
{
   wait();
}

void FractalCalculationThread::run()
{
   const unsigned int width  = Image->width();
   const unsigned int height = Image->height();
   
   for(unsigned int step = ProgStep;step >= 1;step /= 2) {
      for(unsigned int y = 0;y < height;y += step) {
         for(unsigned int x = 0;x < width;x += step) {
            if(Buffer->getPoint(x, y) == (unsigned int)~0) {
               const unsigned int value = Algorithm->calculatePoint(x, y);
               Buffer->setPoint(x, y, value);

               const unsigned int rgb = ColorScheme->getColor(value);
               for(unsigned int vy = y;vy < MIN(y + step, height);vy++) {
                  for(unsigned int vx = x;vx < MIN(x + step, width);vx++) {
                     Image->setPixel(vx, vy, rgb);
                  }
               }
               if(Stop) {
                 puts("STOP!!!!");
                  goto finished;
               }
            }
         }
      }
puts("Ebene beendet!");      
      postEvent(Parent, new QEvent(QEvent::User));
   }
finished:
puts("Berechnung beendet!");
   postEvent(Parent, new QEvent((QEvent::Type)(QEvent::User + 1)));
}

void FractalCalculationThread::stop()
{
   Stop = true;
   wait();
   Stop = false;
}
