/***************************************************************************
                          mandelbrotn.cpp  -  description
                             -------------------
    begin                : Sam Mai 17 2003
    copyright            : (C) 2003 by Thomas Dreibholz
    email                : dreibh@exp-math.uni-essen.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "mandelbrotn.h"
#include "doubleconfigentry.h"
#include <math.h>
#include <iostream>


MandelbrotN* MandelbrotN::Registration = new MandelbrotN();


MandelbrotN::MandelbrotN() : N(2.0)
{
    ConfigEntries.append(new DoubleConfigEntry(&N, "N"));
}

MandelbrotN::~MandelbrotN()
{
}

const char* MandelbrotN::getName()
{
   return("MandelbrotN z[i+1]=z[i]^N-c");
}

unsigned int MandelbrotN::calculatePoint(const unsigned int x,
                                         const unsigned int y)
{
   const std::complex<double> c = std::complex<double>(C1.real() + ((double)x * StepX),
                                                       C1.imag() + ((double)y * StepY));
   std::complex<double> z(0.0, 0.0);
   unsigned int         i;

   for(i = 0;i < MaxIterations;i++) {
      //std::cout << "z vorher" << z*z << std::endl;

      z = pow(z, (int)rint(N));
      //std::cout << "z nachher" << z << std::endl;
      z -= c;
      if(z.real() * z.real() + z.imag() * z.imag() >= 2.0) {
         return(i);
      }
   }
   return(i);
}


std::complex<double> MandelbrotN::defaultC1()
{
   return(std::complex<double>(-1.5, 1.5));
}

std::complex<double> MandelbrotN::defaultC2()
{
   return(std::complex<double>(1.5, -1.5));
}

const char* MandelbrotN::getSaveIdentifier()
{
   return("MandelbrotN");
}
